@@ -108,14 +108,14 @@ int _dl_unmap_cache(void)
 /* This function's behavior must exactly match that 
  * in uClibc/ldso/util/ldd.c */
 static struct elf_resolve * 
-search_for_named_library(char *name, int secure, const char *path_list)
+search_for_named_library(char *name, int secure, const char *path_list,
+	struct dyn_elf **rpnt)
 {
 	int i, count = 1;
 	char *path, *path_n;
 	char mylibname[2050];
 	struct elf_resolve *tpnt1;
 
-
 	/* We need a writable copy of this string */
 	path = _dl_strdup(path_list);
 	if (!path) {
@@ -140,8 +140,11 @@ search_for_named_library(char *name, int secure, const char *path_list)
 		_dl_strcpy(mylibname, path_n); 
 		_dl_strcat(mylibname, "/"); 
 		_dl_strcat(mylibname, name);
-		if ((tpnt1 = _dl_load_elf_shared_library(secure, mylibname, 0)) != NULL)
-		    return tpnt1;
+		if ((tpnt1 = _dl_load_elf_shared_library(secure, rpnt,
+			mylibname)) != NULL)
+		{
+			return tpnt1;
+		}
 		path_n += (_dl_strlen(path_n) + 1);
 	}
 	return NULL;
@@ -156,7 +159,7 @@ unsigned long _dl_error_number;
 unsigned long _dl_internal_error_number;
 extern char *_dl_ldsopath;
 
-struct elf_resolve *_dl_load_shared_library(int secure, 
+struct elf_resolve *_dl_load_shared_library(int secure, struct dyn_elf **rpnt,
 	struct elf_resolve *tpnt, char *full_libname)
 {
 	char *pnt;
@@ -185,7 +188,7 @@ struct elf_resolve *_dl_load_shared_library(int secure,
 	   /usr/i486-sysv4/lib for /usr/lib in library names. */
 
 	if (libname != full_libname) {
-		tpnt1 = _dl_load_elf_shared_library(secure, full_libname, 0);
+		tpnt1 = _dl_load_elf_shared_library(secure, rpnt, full_libname);
 		if (tpnt1)
 			return tpnt1;
 		goto goof;
@@ -204,7 +207,7 @@ struct elf_resolve *_dl_load_shared_library(int secure,
 #ifdef DL_DEBUG
 				_dl_dprintf(2, "searching RPATH: '%s'\n", pnt);
 #endif
-				if ((tpnt1 = search_for_named_library(libname, secure, pnt)) != NULL) 
+				if ((tpnt1 = search_for_named_library(libname, secure, pnt, rpnt)) != NULL) 
 				{
 				    return tpnt1;
 				}
@@ -217,7 +220,7 @@ struct elf_resolve *_dl_load_shared_library(int secure,
 #ifdef DL_DEBUG
 	    _dl_dprintf(2, "searching _dl_library_path: '%s'\n", _dl_library_path);
 #endif
-	    if ((tpnt1 = search_for_named_library(libname, secure, _dl_library_path)) != NULL) 
+	    if ((tpnt1 = search_for_named_library(libname, secure, _dl_library_path, rpnt)) != NULL) 
 	    {
 		return tpnt1;
 	    }
@@ -240,7 +243,7 @@ struct elf_resolve *_dl_load_shared_library(int secure,
 				 libent[i].flags == LIB_ELF_LIBC5) &&
 				_dl_strcmp(libname, strs + libent[i].sooffset) == 0 &&
 				(tpnt1 = _dl_load_elf_shared_library(secure, 
-				     strs + libent[i].liboffset, 0)))
+				     rpnt, strs + libent[i].liboffset)))
 				return tpnt1;
 		}
 	}
@@ -251,7 +254,7 @@ struct elf_resolve *_dl_load_shared_library(int secure,
 #ifdef DL_DEBUG
 	_dl_dprintf(2, "searching in ldso dir: %s\n", _dl_ldsopath);
 #endif
-	if ((tpnt1 = search_for_named_library(libname, secure, _dl_ldsopath)) != NULL) 
+	if ((tpnt1 = search_for_named_library(libname, secure, _dl_ldsopath, rpnt)) != NULL) 
 	{
 	    return tpnt1;
 	}
@@ -268,7 +271,7 @@ struct elf_resolve *_dl_load_shared_library(int secure,
 			UCLIBC_DEVEL_PREFIX "/lib:"
 			UCLIBC_BUILD_DIR "/lib:"
 			"/usr/lib:"
-			"/lib")
+			"/lib", rpnt)
 		    ) != NULL) 
 	{
 	    return tpnt1;
@@ -286,16 +289,15 @@ goof:
 	return NULL;
 }
 
+
 /*
  * Read one ELF library into memory, mmap it into the correct locations and
  * add the symbol info to the symbol chain.  Perform any relocations that
  * are required.
  */
 
-//extern _elf_rtbndr(void);
-
-struct elf_resolve *_dl_load_elf_shared_library(int secure, 
-	char *libname, int flag)
+struct elf_resolve *_dl_load_elf_shared_library(int secure,
+	struct dyn_elf **rpnt, char *libname)
 {
 	elfhdr *epnt;
 	unsigned long dynamic_addr = 0;
@@ -311,14 +313,24 @@ struct elf_resolve *_dl_load_elf_shared_library(int secure,
 	unsigned long *lpnt;
 	unsigned long libaddr;
 	unsigned long minvma = 0xffffffff, maxvma = 0;
-
 	int i;
 	int infile;
 
 	/* If this file is already loaded, skip this step */
 	tpnt = _dl_check_hashed_files(libname);
-	if (tpnt)
+	if (tpnt) {
+		if (*rpnt) {
+			(*rpnt)->next = (struct dyn_elf *)
+				_dl_malloc(sizeof(struct dyn_elf));
+			_dl_memset((*rpnt)->next, 0, sizeof(*((*rpnt)->next)));
+			*rpnt = (*rpnt)->next;
+			(*rpnt)->dyn = tpnt;
+			tpnt->symbol_scope = _dl_symbol_tables;
+		}
+		tpnt->usage_count++;
+		tpnt->libtype = elf_lib;
 		return tpnt;
+	}
 
 	/* If we are in secure mode (i.e. a setu/gid binary using LD_PRELOAD),
 	   we don't load the library if it isn't setuid. */
@@ -438,9 +450,9 @@ struct elf_resolve *_dl_load_elf_shared_library(int secure,
 				char *cpnt;
 
 				status = (char *) _dl_mmap((char *) ((piclib ? libaddr : 0) + 
-					(ppnt->p_vaddr & 0xfffff000)), (ppnt->p_vaddr & 0xfff) 
+					(ppnt->p_vaddr & PAGE_ALIGN)), (ppnt->p_vaddr & ADDR_ALIGN) 
 					+ ppnt->p_filesz, LXFLAGS(ppnt->p_flags), flags, infile, 
-					ppnt->p_offset & 0x7ffff000);
+					ppnt->p_offset & OFFS_ALIGN);
 
 				if (_dl_mmap_check_error(status)) {
 					_dl_dprintf(2, "%s: can't map '%s'\n", 
@@ -452,9 +464,9 @@ struct elf_resolve *_dl_load_elf_shared_library(int secure,
 				};
 
 				/* Pad the last page with zeroes. */
-				cpnt = (char *) (status + (ppnt->p_vaddr & 0xfff) +
+				cpnt = (char *) (status + (ppnt->p_vaddr & ADDR_ALIGN) +
 							  ppnt->p_filesz);
-				while (((unsigned long) cpnt) & 0xfff)
+				while (((unsigned long) cpnt) & ADDR_ALIGN)
 					*cpnt++ = 0;
 
 				/* I am not quite sure if this is completely
@@ -463,17 +475,17 @@ struct elf_resolve *_dl_load_elf_shared_library(int secure,
 				 * /dev/zero if there are any pages left over
 				 * that are not mapped as part of the file */
 
-				map_size = (ppnt->p_vaddr + ppnt->p_filesz + 0xfff) & 0xfffff000;
+				map_size = (ppnt->p_vaddr + ppnt->p_filesz + ADDR_ALIGN) & PAGE_ALIGN;
 				if (map_size < ppnt->p_vaddr + ppnt->p_memsz)
 					status = (char *) _dl_mmap((char *) map_size + 
 						(piclib ? libaddr : 0), 
 						ppnt->p_vaddr + ppnt->p_memsz - map_size, 
 						LXFLAGS(ppnt->p_flags), flags | MAP_ANONYMOUS, -1, 0);
 			} else
-				status = (char *) _dl_mmap((char *) (ppnt->p_vaddr & 0xfffff000) 
-					+ (piclib ? libaddr : 0), (ppnt->p_vaddr & 0xfff) + 
+				status = (char *) _dl_mmap((char *) (ppnt->p_vaddr & PAGE_ALIGN) 
+					+ (piclib ? libaddr : 0), (ppnt->p_vaddr & ADDR_ALIGN) + 
 					ppnt->p_filesz, LXFLAGS(ppnt->p_flags), flags, 
-					infile, ppnt->p_offset & 0x7ffff000);
+					infile, ppnt->p_offset & OFFS_ALIGN);
 			if (_dl_mmap_check_error(status)) {
 				_dl_dprintf(2, "%s: can't map '%s'\n", _dl_progname, libname);
 				_dl_internal_error_number = DL_ERROR_MMAP_FAILED;
@@ -514,7 +526,22 @@ struct elf_resolve *_dl_load_elf_shared_library(int secure,
 
 	dynamic_size = dynamic_size / sizeof(Elf32_Dyn);
 	_dl_memset(dynamic_info, 0, sizeof(dynamic_info));
-	for (i = 0; i < dynamic_size; i++) {
+
+#if defined(__mips__)
+	{
+		int i = 1;
+		Elf32_Dyn *dpnt = (Elf32_Dyn *) dynamic_addr;
+
+		while(dpnt->d_tag) {
+			dpnt++;
+			i++;
+		}
+		dynamic_size = i;
+	}
+#endif
+
+	for (i = 0; i < dynamic_size; i++) 
+	{
 		if (dpnt->d_tag > DT_JMPREL) {
 			dpnt++;
 			continue;
@@ -526,27 +553,40 @@ struct elf_resolve *_dl_load_elf_shared_library(int secure,
 	};
 
 	/* If the TEXTREL is set, this means that we need to make the pages
-	   writable before we perform relocations.  Do this now. They get set back
-	   again later. */
+	   writable before we perform relocations.  Do this now. They get set
+	   back again later. */
 
 	if (dynamic_info[DT_TEXTREL]) {
 		ppnt = (elf_phdr *) & header[epnt->e_phoff];
 		for (i = 0; i < epnt->e_phnum; i++, ppnt++) {
 			if (ppnt->p_type == PT_LOAD && !(ppnt->p_flags & PF_W))
 				_dl_mprotect((void *) ((piclib ? libaddr : 0) + 
-					    (ppnt->p_vaddr & 0xfffff000)), 
-					(ppnt->p_vaddr & 0xfff) + (unsigned long) ppnt->p_filesz, 
+					    (ppnt->p_vaddr & PAGE_ALIGN)), 
+					(ppnt->p_vaddr & ADDR_ALIGN) + (unsigned long) ppnt->p_filesz, 
 					PROT_READ | PROT_WRITE | PROT_EXEC);
 		}
 	}
 
-
 	tpnt = _dl_add_elf_hash_table(libname, (char *) libaddr, dynamic_info, 
 		dynamic_addr, dynamic_size);
 
 	tpnt->ppnt = (elf_phdr *) (tpnt->loadaddr + epnt->e_phoff);
 	tpnt->n_phent = epnt->e_phnum;
 
+	/*
+	 * Add this object into the symbol chain
+	 */
+	if (*rpnt) {
+		(*rpnt)->next = (struct dyn_elf *)
+			_dl_malloc(sizeof(struct dyn_elf));
+		_dl_memset((*rpnt)->next, 0, sizeof(*((*rpnt)->next)));
+		*rpnt = (*rpnt)->next;
+		(*rpnt)->dyn = tpnt;
+		tpnt->symbol_scope = _dl_symbol_tables;
+	}
+	tpnt->usage_count++;
+	tpnt->libtype = elf_lib;
+
 	/*
 	 * OK, the next thing we need to do is to insert the dynamic linker into
 	 * the proper entry in the GOT so that the PLT symbols can be properly
