@@ -2,9 +2,8 @@
 /*
  * A small little ldd implementation for uClibc
  *
- * Copyright (C) 2000 by Lineo, inc.
- * Copyright (C) 2000,2001 Erik Andersen <andersee@debian.org>
- * Written by Erik Andersen <andersee@debian.org>
+ * Copyright (C) 2000 by Lineo, inc and Erik Andersen
+ * Copyright (C) 2000-2002 Erik Andersen <andersee@debian.org>
  *
  * Several functions in this file (specifically, elf_find_section_type(),
  * elf_find_phdr_type(), and elf_find_dynamic(), were stolen from elflib.c from
@@ -39,6 +38,8 @@
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <endian.h>
+#include <byteswap.h>
 #include "elf.h"
 #ifdef DMALLOC
 #include <dmalloc.h>
@@ -52,15 +53,28 @@ struct library {
 };
 struct library *lib_list = NULL;
 char not_found[] = "not found";
+char *interp = NULL;
+char *interp_dir = NULL;
+int byteswap;
+
+inline uint32_t byteswap32_to_host(uint32_t value)
+{
+	if (byteswap==1) {
+		return(bswap_32(value));
+	} else {
+		return(value);
+	}
+}
 
 
 
 Elf32_Shdr * elf_find_section_type( int key, Elf32_Ehdr *ehdr)
 {
 	int j;
-	Elf32_Shdr *shdr = (Elf32_Shdr *)(ehdr->e_shoff + (char *)ehdr);
+	Elf32_Shdr *shdr;
+	shdr = (Elf32_Shdr *)(ehdr->e_shoff + (char *)ehdr);
 	for (j = ehdr->e_shnum; --j>=0; ++shdr) {
-		if (shdr->sh_type == key) {
+		if (key==byteswap32_to_host(shdr->sh_type)) {
 			return shdr;
 		}
 	}
@@ -72,7 +86,7 @@ Elf32_Phdr * elf_find_phdr_type( int type, Elf32_Ehdr *ehdr)
 	int j;
 	Elf32_Phdr *phdr = (Elf32_Phdr *)(ehdr->e_phoff + (char *)ehdr);
 	for (j = ehdr->e_phnum; --j>=0; ++phdr) {
-		if (type==phdr->p_type) {
+		if (type==byteswap32_to_host(phdr->p_type)) {
 			return phdr;
 		}
 	}
@@ -84,19 +98,19 @@ void * elf_find_dynamic(int const key, Elf32_Dyn *dynp,
 	Elf32_Ehdr *ehdr, int return_val)
 {
 	Elf32_Phdr *pt_text = elf_find_phdr_type(PT_LOAD, ehdr);
-	unsigned tx_reloc = pt_text->p_vaddr - pt_text->p_offset;
-	for (; DT_NULL!=dynp->d_tag; ++dynp) {
-		if (dynp->d_tag == key) {
+	unsigned tx_reloc = byteswap32_to_host(pt_text->p_vaddr) - byteswap32_to_host(pt_text->p_offset);
+	for (; DT_NULL!=byteswap32_to_host(dynp->d_tag); ++dynp) {
+		if (key == byteswap32_to_host(dynp->d_tag)) {
 			if (return_val == 1)
-				return (void *)dynp->d_un.d_val;
+				return (void *)byteswap32_to_host(dynp->d_un.d_val);
 			else
-				return (void *)(dynp->d_un.d_val - tx_reloc + (char *)ehdr );
+				return (void *)(byteswap32_to_host(dynp->d_un.d_val) - tx_reloc + (char *)ehdr );
 		}
 	}
 	return NULL;
 }
 
-int check_elf_header(Elf32_Ehdr const *const ehdr)
+int check_elf_header(Elf32_Ehdr *const ehdr)
 {
 	if (! ehdr || strncmp((void *)ehdr, ELFMAG, SELFMAG) != 0 ||  
 			ehdr->e_ident[EI_CLASS] != ELFCLASS32 ||
@@ -104,6 +118,30 @@ int check_elf_header(Elf32_Ehdr const *const ehdr)
 	{
 		return 1;
 	}
+
+	/* Check if the target endianness matches the host's endianness */
+	byteswap = 0;
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+	if (ehdr->e_ident[5] == ELFDATA2MSB) {
+		/* Ick -- we will have to byte-swap everything */
+		byteswap = 1;
+	}
+#elif __BYTE_ORDER == __BIG_ENDIAN
+	if (ehdr->e_ident[5] == ELFDATA2LSB) {
+		byteswap = 1;
+	}
+#else
+#error Unknown host byte order!
+#endif
+	/* Be vary lazy, and only byteswap the stuff we use */
+	if (byteswap==1) {
+		ehdr->e_type=bswap_16(ehdr->e_type);
+		ehdr->e_phoff=bswap_32(ehdr->e_phoff);
+		ehdr->e_shoff=bswap_32(ehdr->e_shoff);
+		ehdr->e_phnum=bswap_16(ehdr->e_phnum);
+		ehdr->e_shnum=bswap_16(ehdr->e_shnum);
+	}
+
 	return 0;
 }
 
@@ -111,7 +149,7 @@ int check_elf_header(Elf32_Ehdr const *const ehdr)
  * in uClibc/ldso/d-link/readelflib1.c */
 static void search_for_named_library(char *name, char *result, const char *path_list)
 {
-	int i, count = 0;
+	int i, count = 1;
 	char *path, *path_n;
 	struct stat filestat;
 
@@ -135,11 +173,9 @@ static void search_for_named_library(char *name, char *result, const char *path_
 			count++;
 		}
 	}
-
 	path_n = path;
 	for (i = 0; i < count; i++) {
-		*result = '\0';
-		strcat(result, path_n); 
+		strcpy(result, path_n); 
 		strcat(result, "/"); 
 		strcat(result, name);
 		if (stat (result, &filestat) == 0 && filestat.st_mode & S_IRUSR) {
@@ -152,8 +188,8 @@ static void search_for_named_library(char *name, char *result, const char *path_
 	*result = '\0';
 }
 
-void locate_library_file(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, int is_suid, 
-		struct library *lib, char *ldpath)
+void locate_library_file(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, 
+		int is_suid, struct library *lib)
 {
 	char *buf;
 	char *path;
@@ -200,18 +236,22 @@ void locate_library_file(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, int
 		}
 	}
 
-	if (ldpath) {
-		search_for_named_library(lib->name, buf, ldpath);
+#ifdef USE_CACHE
+	/* FIXME -- add code to check the Cache here */ 
+#endif
+
+
+	/* Next look for libraries wherever the shared library 
+	 * loader was installed -- this is usually where we
+	 * should find things... */
+	if (interp_dir) {
+		search_for_named_library(lib->name, buf, interp_dir);
 		if (*buf != '\0') {
 			lib->path = buf;
 			return;
 		}
 	}
 
-#ifdef USE_CACHE
-	/* FIXME -- add code to check the Cache here */ 
-#endif
-
 	/* Lastly, search the standard list of paths for the library.
 	   This list must exactly match the list in uClibc/ldso/d-link/readelflib1.c */
 	path =	UCLIBC_TARGET_PREFIX "/usr/lib:"
@@ -229,7 +269,7 @@ void locate_library_file(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, int
 	}
 }
 
-static int add_library(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, int is_setuid, char *s, char *ldpath)
+static int add_library(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, int is_setuid, char *s)
 {
 	char *tmp, *tmp1, *tmp2;
 	struct library *cur, *newlib=lib_list;
@@ -273,7 +313,7 @@ static int add_library(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, int i
 	newlib->next = NULL;
 
 	/* Now try and locate where this library might be living... */
-	locate_library_file(ehdr, dynamic, strtab, is_setuid, newlib, ldpath);
+	locate_library_file(ehdr, dynamic, strtab, is_setuid, newlib);
 
 	//printf("add_library is adding '%s' to '%s'\n", newlib->name, newlib->path);
 	if (!lib_list) {
@@ -288,20 +328,12 @@ static int add_library(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, int i
 
 static void find_needed_libraries(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *strtab, int is_setuid)
 {
-	char * ldpath = NULL;
 	Elf32_Dyn  *dyns;
-	Elf32_Phdr *phdr;
 
-	/* Find the path where the shared lib loader lives */
-	phdr = elf_find_phdr_type(PT_INTERP, ehdr);
-	if (phdr) {
-		ldpath = strdup((char*)ehdr + phdr->p_offset);
-		ldpath = basename(ldpath);
-	}
-
-	for (dyns=dynamic; dyns->d_tag!=DT_NULL; ++dyns) {
-		if (dyns->d_tag == DT_NEEDED) {
-			add_library(ehdr, dynamic, strtab, is_setuid, (char*)strtab + dyns->d_un.d_val, ldpath);
+	for (dyns=dynamic; byteswap32_to_host(dyns->d_tag)!=DT_NULL; ++dyns) {
+		if (DT_NEEDED == byteswap32_to_host(dyns->d_tag)) {
+			add_library(ehdr, dynamic, strtab, is_setuid, (char*)strtab + 
+					byteswap32_to_host(dyns->d_un.d_val));
 		}
 	}
 }
@@ -317,9 +349,18 @@ static void find_elf_interpreter(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *str
 	phdr = elf_find_phdr_type(PT_INTERP, ehdr);
 	if (phdr) {
 		struct library *cur, *newlib=NULL;
-		char *s = (char*)ehdr + phdr->p_offset;
+		char *s = (char*)ehdr + byteswap32_to_host(phdr->p_offset);
 	
 		char *tmp, *tmp1;
+		interp = strdup(s);
+		interp_dir = strdup(s);
+		tmp = strrchr(interp_dir, '/');
+		if (*tmp)
+			*tmp = '\0';
+		else {
+			free(interp_dir);
+			interp_dir = interp;
+		}
 		tmp1 = tmp = s;
 		while (*tmp) {
 			if (*tmp == '/')
@@ -346,6 +387,7 @@ static void find_elf_interpreter(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *str
 		newlib->resolved = 1;
 		newlib->next = NULL;
 	
+#if 0
 		//printf("find_elf_interpreter is adding '%s' to '%s'\n", newlib->name, newlib->path);
 		if (!lib_list) {
 			lib_list = newlib;
@@ -353,6 +395,7 @@ static void find_elf_interpreter(Elf32_Ehdr* ehdr, Elf32_Dyn* dynamic, char *str
 			for (cur = lib_list;  cur->next; cur=cur->next); /* nothing */
 			cur->next = newlib;
 		}
+#endif
 	}
 }
 
@@ -412,12 +455,12 @@ foo:
 	}
 
 	dynsec = elf_find_section_type(SHT_DYNAMIC, ehdr);
+	find_elf_interpreter(ehdr, dynamic, dynstr, is_suid);
 	if (dynsec) {
-		dynamic = (Elf32_Dyn*)(dynsec->sh_offset + (int)ehdr);
+		dynamic = (Elf32_Dyn*)(byteswap32_to_host(dynsec->sh_offset) + (int)ehdr);
 		dynstr = (char *)elf_find_dynamic(DT_STRTAB, dynamic, ehdr, 0);
 		find_needed_libraries(ehdr, dynamic, dynstr, is_suid);
 	}
-	find_elf_interpreter(ehdr, dynamic, dynstr, is_suid);
 	
 	return 0;
 }
@@ -458,6 +501,8 @@ int main( int argc, char** argv)
 		got_em_all=1;
 		printf("\t%s => %s\n", cur->name, cur->path);
 	}
+	if (interp_dir && got_em_all==1)
+		printf("\t%s => %s\n", interp, interp);
 	if (got_em_all==0)
 		printf("\tnot a dynamic executable\n");
 
