@@ -12,6 +12,12 @@
 #include <sys/wait.h>
 #include <errno.h>
 
+/* uClinux-2.0 has vfork, but Linux 2.0 doesn't */
+#include <sys/syscall.h>
+#if ! defined __NR_vfork && defined __UCLIBC_HAS_MMU__ 
+#define vfork fork	
+#endif
+
 FILE *popen (const char *command, const char *mode)
 {
 	FILE *fp;
@@ -28,8 +34,8 @@ FILE *popen (const char *command, const char *mode)
 		if ((fp = fdopen(pnr, mode)) != NULL) {
 			if ((pid = vfork()) == 0) {	/* vfork -- child */
 				close(pnr);
-				close(reading);
 				if (pr != reading) {
+					close(reading);
 					dup2(pr, reading);
 					close(pr);
 				}
